require "mandel"
require 'phi'
include Phi

DefaultMaxDepth = 50
DefaultSX = -2.25
DefaultSY = 1.75
DefaultEX = 1.25
DefaultEY = -1.75
Width = 400
Height = 400

def reset
  $max_depth = DefaultMaxDepth
  $s_re = DefaultSX
  $s_im = DefaultSY
  $e_re = DefaultEX
  $e_im = DefaultEY
  $dx = ($e_re - $s_re).abs / Width
  $dy = ($e_im - $s_im).abs / Height
end


def zoom(a, b, c, d)
  center_x = (a + c) / 2
  center_y = (b + d) / 2
  size = (c - a).abs
  size = (d - b).abs if (size < (d - b).abs)
  size = 1 if (size < 1)
  zoom_rate = ((Width + Height) / 2).to_f / size
  $max_depth = ($max_depth.to_f * Math.sqrt(Math.sqrt(Math.sqrt(zoom_rate)))).to_i

  move_x_rate = (center_x - (Width / 2)).to_f / (Width / 2)
  move_y_rate = (center_y - (Height / 2)).to_f / (Height / 2)

  center_re = ($s_re + $e_re) / 2
  center_im = ($s_im + $e_im) / 2
  c_size_re = ($e_re - $s_re).abs
  c_size_im = ($e_im - $s_im).abs

  center_re = center_re + (move_x_rate * (c_size_re / 2))
  center_im = center_im - (move_y_rate * (c_size_im / 2))

  $s_re = center_re - ((c_size_re / 2) / zoom_rate)
  $s_im = center_im + ((c_size_im / 2) / zoom_rate)
  $e_re = center_re + ((c_size_re / 2) / zoom_rate)
  $e_im = center_im - ((c_size_im / 2) / zoom_rate)

  $dx = ($e_re - $s_re).abs / Width
  $dy = ($e_im - $s_im).abs / Height
end


def mandel(x, y)
  re = $s_re + ($dx * x)
  im = $s_im - ($dy * y)
  return Mandel.mandel(re, im, $max_depth)
end


reset()
$calc_y = StartCalcY = 0
$calc_on = true

def clear
  $calc_y = StartCalcY
end

$start_x = $start_y = 0
$current_rect = nil


afm = <<EOL
object Form1: TForm
  Left = 192
  Top = 114
  Width = 400
  Height = 400
  Caption = 'a_mandel'
  Color = clBtnFace
  Font.Charset = SHIFTJIS_CHARSET
  Font.Color = clWindowText
  Font.Height = -12
  Font.Name = #65325#65331' '#65328#12468#12471#12483#12463
  Font.Style = []
  OldCreateOrder = False
  PixelsPerInch = 96
  TextHeight = 12
end
EOL

form = Component._load afm

pbox = PaintBox.new form
pbox.align = AL_CLIENT

Phi.new_popup_menu form, :popup_menu, [
  Phi.new_item('&Paint', 'Ctrl+P', :mi_paint),
  Phi.new_item('&Reset', 'Ctrl+R', :mi_reset),
  Phi.new_item('&Screen Shot', 'Ctrl+S', :mi_screen_shot),
]

module Phi
  module MenuContainer
    def popup_menu_attr_flatten
      define_child_attr(popup_menu.items)
    end
  end
end
form.popup_menu_attr_flatten

def form.on_context_popup(handled, point)
  popup_menu.popup(left+point.x, top+point.y)
  true
end

form.mi_paint.on_click = proc do
  pbox.do_paint
end

form.mi_screen_shot.on_click = proc do
  bmap = Bitmap.new
  bmap.width = pbox.width
  bmap.height = pbox.height
  bmap.canvas.copy_rect pbox.rect, pbox.canvas, pbox.rect
  CLIPBOARD.assign bmap
end

def pbox.do_paint
  canvas.pen.color = 0x00ffffff
  canvas.ellipse(50,50,100,100)
  canvas.poly_bezier(0,100,80,60,100,200,320,100)

  for x in 0 .. Width - 1
    for y in 0 .. Height - 1
      depth = mandel(x, y)
      if depth >= $max_depth
        canvas[x,y] = 0
      else
        canvas[x,y] = 0x00ffffff - (0x00ffffff * depth / $max_depth)
      end
    end
  end
end

def pbox.start
  @select_rect = nil
  do_paint
end

form.mi_reset.on_click = proc do
  form.caption = 'reset'
  reset
  pbox.do_paint
end

class SelectRect < Rect
  def move(x,y)
    if self.left < x then self.right  = x else self.left = x end
    if self.top  < y then self.bottom = y else self.top  = y end
  end
  
  def xyxy
    return self.left, self.top, self.right, self.bottom
  end
  
  def to_str
    "(%d,%d)-(%d,%d)" % xyxy
  end
end

def pbox.on_mouse_down(btn,sft,x,y)
  if btn == MB_LEFT
    @select_rect = SelectRect.new(x,y,x,y)
  end
end

def pbox.on_mouse_move(sft,x,y)
  parent.caption = sprintf 'a_mandel (x=%d, y=%d)', x,y
  if @select_rect
    canvas.draw_focus_rect @select_rect
    @select_rect.move x,y
    canvas.draw_focus_rect @select_rect
  end
end

def pbox.on_mouse_up(btn,sft,x,y)
  return unless @select_rect
  canvas.draw_focus_rect @select_rect
  parent.caption = 'zoom... ' + @select_rect
  zoom(*(@select_rect.xyxy))
  @select_rect = nil
  do_paint
end

form.show
pbox.start
Phi.mainloop
